@@ -2,6 +2,8 @@ import argparse
 import json
 import sys
 import os
+from pathlib import Path
+from pprint import pformat
 
 import datetime
 import logging
@@ -108,7 +110,6 @@ class Terminal(QtWidgets.QMainWindow):
         node (:class:`~.networking.Net_Node`): Our Net_Node we use to communicate with our main networking object
         networking (:class:`~.networking.Terminal_Station`): Our networking object to communicate with the outside world
         subjects (dict): A dictionary mapping subject ID to :class:`~.subject.Subject` object.
-        pilots (dict): A dictionary mapping pilot ID to a list of its subjects, its IP, and any other pilot attributes.
         layout (:class:`QtWidgets.QGridLayout`): Layout used to organize widgets
         control_panel (:class:`~.gui.Control_Panel`): Control Panel to manage pilots and subjects
         data_panel (:class:`~.plots.Plot_Widget`): Plots for each pilot and subject.
@@ -130,7 +131,6 @@ class Terminal(QtWidgets.QMainWindow):
 
         # data
         self.subjects = {}  # Dict of our open subject objects
-        self.pilots = None
 
         # gui
         self.layout = None
@@ -141,13 +141,12 @@ class Terminal(QtWidgets.QMainWindow):
         self.data_panel = None
         self.logo = None
 
+        # property private attributes
+        self._pilots = None
+
         # logging
         self.logger = init_logger(self)
 
-        # Load pilots db as ordered dictionary
-        with open(prefs.get('PILOT_DB')) as pilot_file:
-            self.pilots = json.load(pilot_file, object_pairs_hook=odict)
-
         # Listen dictionary - which methods to call for different messages
         # Methods are spawned in new threads using handle_message
         self.listens = {
@@ -378,6 +377,93 @@ class Terminal(QtWidgets.QMainWindow):
         self.setCentralWidget(self.widget)
         self.initUI()
 
+    ################
+    # Properties
+
+    @property
+    def pilots(self) -> odict:
+        """
+        A dictionary mapping pilot ID to its attributes, including a list of its subjects assigned to it, its IP, etc.
+
+        Returns:
+            dict: like ``self.pilots['pilot_id'] = {'subjects': ['subject_0', 'subject_1'], 'ip': '192.168.0.101'}``
+        """
+
+        # try to load, if none exists make one
+        if self._pilots is None:
+
+            pilot_db_fn = Path(prefs.get('PILOT_DB'))
+
+            # if pilot file doesn't exist, make blank one
+            if not pilot_db_fn.exists():
+                self.logger.warning(f'No pilot_db.json file was found at {pilot_db_fn}, creating a new one')
+                self._pilots = odict()
+                with open(pilot_db_fn, 'w') as pilot_file:
+                    json.dump(self._pilots, pilot_file)
+
+            # otherwise, try to load it
+            else:
+                try:
+                    # Load pilots db as ordered dictionary
+                    with open(pilot_db_fn, 'r') as pilot_file:
+                        self._pilots = json.load(pilot_file, object_pairs_hook=odict)
+                    self.logger.info(f'successfully loaded pilot_db.json file from {pilot_db_fn}')
+                    self.logger.debug(pformat(self._pilots))
+                except Exception as e:
+                    self.logger.exception((f"Exception opening pilot_db.json file at {pilot_db_fn}, got exception: {e}.\n",
+                                           "Not proceeding to prevent possibly overwriting corrupt pilot_db.file"))
+                    raise e
+
+        return self._pilots
+
+
+    @property
+    def protocols(self) -> list:
+        """
+        List of protocol names available in ``PROTOCOLDIR``
+
+        Returns:
+            list: list of protocol names in ``prefs.get('PROTOCOLDIR')``
+        """
+        # get list of protocol files
+        protocols = os.listdir(prefs.get('PROTOCOLDIR'))
+        protocols = [os.path.splitext(p)[0] for p in protocols if p.endswith('.json')]
+        return protocols
+
+    @property
+    def subject_protocols(self) -> dict:
+        """
+
+        Returns:
+            subject_protocols (dict): a dictionary of subjects: [protocol, step]
+        """
+        # get subjects and current protocols
+        subjects = self.subject_list
+        subjects_protocols = {}
+        for subject in subjects:
+            if subject not in self.subjects.keys():
+                self.subjects[subject] = Subject(subject)
+
+            subjects_protocols[subject] = [self.subjects[subject].protocol_name, self.subjects[subject].step]
+
+        return subjects_protocols
+
+    @property
+    def subject_list(self) -> list:
+        """
+        Get a list of all subject IDs
+
+        Returns:
+            list: list of all subject IDs present in :attr:`.Terminal.pilots`
+        """
+        subjects = []
+        for pilot, vals in self.pilots.items():
+            subjects.extend(vals['subjects'])
+
+        # use sets to get a unique list
+        subjects = list(set(subjects))
+
+        return subjects
 
     ##########################3
     # Listens & inter-object methods
@@ -626,23 +712,6 @@ class Terminal(QtWidgets.QMainWindow):
                 with open(protocol_file, 'w') as pfile_open:
                     json.dump(save_steps, pfile_open, indent=4, separators=(',', ': '), sort_keys=True)
 
-    @property
-    def subject_list(self):
-        """
-        Get a list of all subject IDs
-
-        Returns:
-            list: list of all subject IDs present in :attr:`.Terminal.pilots`
-        """
-        subjects = []
-        for pilot, vals in self.pilots.items():
-            subjects.extend(vals['subjects'])
-
-        # use sets to get a unique list
-        subjects = list(set(subjects))
-
-        return subjects
-
     def subject_weights(self):
         """
         Gets recent weights from all :attr:`~.Terminal.subjects` and
@@ -692,36 +761,6 @@ class Terminal(QtWidgets.QMainWindow):
         msgbox.setDetailedText("\n".join(sorted(updated_subjects)))
         msgbox.exec_()
 
-    @property
-    def protocols(self):
-        """
-        Returns:
-            list: list of protocol files in ``prefs.get('PROTOCOLDIR')``
-        """
-        # get list of protocol files
-        protocols = os.listdir(prefs.get('PROTOCOLDIR'))
-        protocols = [os.path.splitext(p)[0] for p in protocols if p.endswith('.json')]
-        return protocols
-
-    @property
-    def subject_protocols(self):
-        """
-
-        Returns:
-            subject_protocols (dict): a dictionary of subjects: [protocol, step]
-        """
-        # get subjects and current protocols
-        subjects = self.subject_list
-        subjects_protocols = {}
-        for subject in subjects:
-            if subject not in self.subjects.keys():
-                self.subjects[subject] = Subject(subject)
-
-            subjects_protocols[subject] = [self.subjects[subject].protocol_name, self.subjects[subject].step]
-
-        return subjects_protocols
-
-
     def reassign_protocols(self):
         """
         Batch reassign protocols and steps.
