@@ -1,5 +1,5 @@
 from autopilot.tasks.task import Task
-from autopilot.tasks.nafc import Nafc, Nafc_Gap
+from autopilot.tasks.nafc import Nafc, Nafc_Gap, Nafc_Gap_Laser
 from autopilot.tasks.gonogo import GoNoGo
 from autopilot.tasks.parallax import Parallax
 from autopilot.tasks.free_water import Free_Water
@@ -10,6 +10,7 @@ from autopilot.tasks.test import DLC_Latency, DLC_Hand
 
 TASK_LIST = {'2AFC':Nafc,
              '2AFC_Gap':Nafc_Gap,
+             '2AFC_Gap_Laser':Nafc_Gap_Laser,
              'Free Water':Free_Water,
              'GoNoGo': GoNoGo,
              'Parallax': Parallax,
